package rattrapage;

import java.io.Console;
import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import rattrapage.donnees.Etat;
import rattrapage.donnees.Livre;

public class Application {
  public static final int CHOIX_MAX = 5;

  /**
   * Fonction principale permettant de lancer le programme.
   *
   * @param args : arguments de la ligne de commande ; rien d'attendu.
   */
  public static void main(String... args) {
    Etat etat = new Etat();
    var traitements = initTraitements();
    while (!etat.isTermine()) {
      afficher_menu();
      var choix = lire_clavier();
      traitements.get(choix - 1).accept(etat);
    }
  }
    
  /**
   * Fonction créant la liste des traitements accessibles depuis le menu.
   *
   * @return la liste des traitements
   */
  public static List<Consumer<Etat>> initTraitements() {
    List<Consumer<Etat>> traitements = new ArrayList<>();
    Consumer<Etat> traiteQuitter = etat -> etat.arreter();
    Consumer<Etat> traiteLister = etat -> etat.getBibliotheque()
        .getBiblio()
        .stream()
        .sorted()
        .forEach(System.out::println);
    Consumer<Etat> traiteAjouter = etat -> etat.getBibliotheque().ajouterLivre(saisirLivre());
    Consumer<Etat> traiteSupprimer = etat -> etat.getBibliotheque().supprimerDernier();
    Consumer<Etat> traite2000 = etat -> etat.getBibliotheque()
        .getBiblio()
        .stream()
        .filter(livre -> livre.getAnnee() < 2000)
        .forEach(System.out::println);
    traitements.add(traiteLister);
    traitements.add(traiteAjouter);
    traitements.add(traite2000);
    traitements.add(traiteSupprimer);
    traitements.add(traiteQuitter);
    return traitements;
  }

  /**
   * Fonction qui affiche le menu des possibilités à l'utilisateur.
   */
  public static void afficher_menu() {
    System.out.println("1. Afficher la liste de livres");
    System.out.println("2. Ajouter un livre");
    System.out.println("3. Afficher les livres publiés avant l'an 2000");
    System.out.println("4. Supprimer le dernier livre ajouté");
    System.out.println("5. Quitter");
  }

  /**
   * Méthode pour lire le choix de l'utilisateur, numérique.
   *
   * @return le choix de l'utilisateur
   */
  public static int lire_clavier() {
    int choix = 0;
    Console console = System.console();
    boolean termine = false;
    while (!termine) {
      console.printf("Votre choix ==> ");
      String saisie = console.readLine();
      try {
        choix = Integer.parseInt(saisie);
        if (choix <= CHOIX_MAX & choix >= 1) {
          termine = true;
        } else {
          System.err.println("Choix non valide");
        }
      } catch (NumberFormatException nfe) {
        System.err.println("Merci de saisir un nombre");
      }
    }
    return choix;
  }

  /**
   * Méthode permettant de demander à l'utilisateur les informations nécessaires
   * à la saisie d'un nouveau livre.
   *
   * @return le livre créé
  */
  public static Livre saisirLivre() {
    Console console = System.console();
    System.out.print("Titre : ");
    String titre = console.readLine();
    System.out.print("Année : ");
    int annee = Integer.parseInt(console.readLine());
    Livre livre = new Livre(titre, annee);
    return livre;
  }
}
